@@ -113,6 +113,14 @@
 	         p1_test_fail (__FILE__, __LINE__, __FUNCTION__, "assertion failed (%s): '%s' does not contain '%s'", \
 	                       #expr, __str, needle); \
 	} while (0)
+#define assert_mem_eq(m1, l1, m2, l2) \
+	do { size_t __l1 = (l1); \
+	     size_t __l2 = (l2); \
+	     const void *__m1 = (m1); \
+	     const void *__m2 = (m2); \
+	     if ((__l1 == 0 && __l2 == 0) || (__l1 == __l2 && __m1 && __m2 && memcmp (__m1, __m2, l1) == 0)) ; else \
+	         p11_test_fail (__FILE__, __LINE__, __FUNCTION__, "assertion failed"); \
+	} while (0)
 
 #endif /* !P11_TEST_SOURCE */
 
