@@ -173,6 +173,8 @@ call_done (rpc_client *module,
            p11_rpc_message *msg,
            CK_RV ret)
 {
+	p11_buffer *buf;
+
 	assert (module != NULL);
 	assert (msg != NULL);
 
@@ -189,9 +191,9 @@ call_done (rpc_client *module,
 
 	/* We used the same buffer for input/output, so this frees both */
 	assert (msg->input == msg->output);
-	p11_rpc_buffer_free (msg->input);
-
+	buf = msg->input;
 	p11_rpc_message_clear (msg);
+	p11_rpc_buffer_free (buf);
 
 	return ret;
 }
@@ -238,16 +240,11 @@ proto_read_attribute_array (p11_rpc_message *msg,
 		CK_ATTRIBUTE temp;
 
 		memset (&temp, 0, sizeof (temp));
-		if (!p11_rpc_buffer_get_attribute (msg->input, &offset, &temp)) {
+		if (!p11_rpc_message_get_attribute (msg, msg->input, &offset, &temp)) {
 			msg->parsed = offset;
 			return PARSE_ERROR;
 		}
 
-		if (IS_ATTRIBUTE_ARRAY (&temp)) {
-			p11_debug("recursive attribute array is not supported");
-			return PARSE_ERROR;
-		}
-
 		/* Try and stuff it in the output data */
 		if (arr) {
 			CK_ATTRIBUTE *attr = &(arr[i]);
@@ -787,10 +784,11 @@ rpc_C_Initialize (CK_X_FUNCTION_LIST *self,
 			assert (module->vtable->disconnect != NULL);
 			(module->vtable->disconnect) (module->vtable, reserved);
 			ret = (module->vtable->connect) (module->vtable, reserved);
-
-			module->version = 0;
-			ret = (module->vtable->authenticate) (module->vtable,
-							      &module->version);
+			if (ret == CKR_OK) {
+				module->version = 0;
+				ret = (module->vtable->authenticate) (module->vtable,
+								      &module->version);
+			}
 		}
 #endif
 	}
